﻿(function (app) {
    app.tutorials = {
        getTutorialsModel: function (tutorialsData) {
            var tutorialsModel = {
                title: "Interactive Tutorials",
                totalProgress: function () {
                    var completedCount = 0;

                    this.items.forEach(function (item) {
                        var isCompleted = item.get("isCompleted").call(item);

                        if (isCompleted) {
                            completedCount++;
                        }

                        return isCompleted;
                    });

                    return Math.round(completedCount * 100 / this.items.length);
                },
                items: []
            };

            tutorialsData.forEach(function (tutorialData) {
                var tutorialModel = app.tutorials._getModel(tutorialData);
                app.lessons.initialize(tutorialModel);
                app.steps.initialize(tutorialModel);

                tutorialsModel.items.push(tutorialModel);
            });

            return kendo.observable(tutorialsModel);
        },
        openTutorialByIndex: function (tutorialIndex) {
            openTutorial.call(app.models.tutorials.items[tutorialIndex]);
        },
        /**
         * @param tutorial
         * @param tutorial.fileName
         * @param tutorial.title
         * @param tutorial.description
         * @param tutorial.estimateTime
         * @param tutorial.index
         * @param tutorial.displayNumber
         * @param tutorial.type - can be "hybrid" or "native"
         *
         * @param tutorial.app
         * @param tutorial.app.projectName
         * @param tutorial.app.repositoryFullName
         *
         * @param tutorial.lessons
         * @param tutorial.lessons[*].title
         * @param tutorial.lessons[*].index
         * @param tutorial.lessons[*].displayNumber
         *
         * @param tutorial.lessons[*].steps
         * @param tutorial.lessons[*].steps[*].title
         * @param tutorial.lessons[*].steps[*].index
         * @param tutorial.lessons[*].steps[*].displayNumber
         * @param tutorial.lessons[*].steps[*].contentRaw
         */
        _getModel: function (data) {
            return {
                metadata: data,
                currentOpenLessonIndex: 0,
                currentLessonIndex: function () {
                    var latestUncompletedLesson;

                    this.lessons.some(function (lessonItem, index) {
                        latestUncompletedLesson = index;

                        return lessonItem.steps.some(function (stepItem) {
                            return !stepItem.get("isCompleted");
                        });
                    });

                    return latestUncompletedLesson;
                },
                currentStepIndex: function () {
                    var latestUncompletedStep = -1;

                    this.lessons.some(function (lessonItem) {
                        return lessonItem.steps.some(function (stepItem, stepIndex) {
                            if (!stepItem.get("isCompleted")) {
                                latestUncompletedStep = stepIndex;
                                return true;
                            }

                            return false;
                        });
                    });

                    // If there is no uncompleted steps in the tutorial return the last step index
                    if (latestUncompletedStep === -1) {
                        latestUncompletedStep = this.lessons[this.lessons.length - 1].steps.length - 1;
                    }

                    return latestUncompletedStep;
                },
                progress: function () {
                    var totalStepsCount = 0,
                        completedStepsCount = 0;

                    totalStepsCount = this.lessons.reduce(function (prevItem, currentItem) {
                        return (prevItem.steps ? prevItem.steps.length : prevItem) + currentItem.steps.length;
                    });

                    this.lessons.every(function (lessonItem) {
                        return lessonItem.steps.every(function (stepItem) {
                            var isCompleted = stepItem.get("isCompleted");
                            if (isCompleted) {
                                completedStepsCount++;
                            }

                            return isCompleted;
                        });
                    });

                    return Math.round(completedStepsCount * 100 / totalStepsCount);
                },
                isCompleted: function () {
                    return this.get("progress").call(this) === 100;
                },
                isStarted: false,
                backgroundImage: function () {
                    return app.ui.getProgressBasedBackgroundImage(this.get("progress").call(this));
                },
                showTutorialStart: function () {
                    return !this.get("isStarted");
                },
                showTutorialContinue: function () {
                    return this.get("isStarted") && !this.get("isCompleted").call(this);
                },
                showTutorialRestart: function () {
                    return this.get("isStarted");
                },
                openTutorial: openTutorial,
                onContinueClick: onTutorialContinue,
                onRestartClick: onTutorialRestart,
                onStartClick: onTutorialStart
            };
        }
    };

    function openTutorial(e) {
        stopPropagation(e);

        // Analytics: TutorialOpen
        app.analyticsBridge.trackTutorialOpen(this.metadata.title, this.metadata.type);

        app.router.navigate(app.routes.getRoute(app.routes.lessons, [this.metadata.index]));
        console.log("open tutorial: " + this.metadata.index);
    }

    function onTutorialStart(e) {
        var overwriteProject = true;

        stopPropagation(e);

        app.projectUtils.openProject(this, overwriteProject)
            .done(startTutorialProgress.bind(this));
    }

    function onTutorialContinue(e) {
        var lesson = this.lessons[this.currentLessonIndex()],
            step = lesson.steps[this.currentStepIndex()],
            overwriteProject = false;

        stopPropagation(e);

        app.projectUtils.openProject(this, overwriteProject)
            .done(function (isNewProject) {
                // Analytics: TutorialContinue
                app.analyticsBridge.trackTutorialContinue(
                    this.metadata.title, this.metadata.type,
                    lesson.metadata.displayNumber, lesson.metadata.title,
                    step.metadata.displayNumber, step.metadata.title);

                if (isNewProject) {
                    restartTutorialProgress.call(this);
                    startTutorialProgress.call(this);
                }
                else {
                    app.router.navigate(app.routes.getRoute(app.routes.steps, [this.metadata.index, this.currentLessonIndex(), this.currentStepIndex()]));
                    console.log("tutorial continued: " + this.metadata.index);
                }
            }.bind(this));
    }

    function onTutorialRestart(e) {
        stopPropagation(e);

        if (!this.isStarted) {
            return;
        }

        app.messageBoxBridge.showWarning(
            "When you restart the tutorial, your project \"" + this.metadata.app.projectName + "\" will be recreated. Please, move your project out of the \"Interactive Tutorial Projects\" folder.",
            "Restart \"" + this.metadata.title + "\" Tutorial")
            .done(function (response) {
                if (response) {
                    restartTutorialProgress.call(this);

                    // The kendo router does not support reload
                    // HACK: change location hash to force the kendo router to navigate to the same view
                    // the navigation must be made outside of the current function so the location hash can update
                    window.location.hash = window.location.hash.substr(0, window.location.hash.length - 2);
                    setTimeout(app.router.navigate.bind(app.router, app.routes.getRoute(app.routes.lessons, [this.metadata.index])));

                    console.log("tutorial restarted: " + this.metadata.index);
                }
            }.bind(this));
    }

    function startTutorialProgress() {
        // Analytics: TutorialStart
        app.analyticsBridge.trackTutorialStart(this.metadata.title, this.metadata.type);

        // ProgressData: AddTutorialData
        app.progressDataBridge.addTutorialData(this.metadata.index);

        this.set("isStarted", true);

        app.router.navigate(app.routes.getRoute(app.routes.steps, [this.metadata.index, 0, 0]));
        console.log("tutorial started: " + this.metadata.index);
    }

    function restartTutorialProgress() {
        var lesson = this.lessons[this.currentLessonIndex()],
            step = lesson.steps[this.currentStepIndex()];

        // Analytics: TutorialRestart
        app.analyticsBridge.trackTutorialRestart(
            this.metadata.title, this.metadata.type,
            lesson.metadata.displayNumber, lesson.metadata.title,
            step.metadata.displayNumber, step.metadata.title);

        // ProgressData: SetCompletedStep
        app.progressDataBridge.removeTutorialData(this.metadata.index);

        this.set("isStarted", false);
        this.lessons.forEach(function (lesson) {
            lesson.steps.forEach(function (step) {
                step.set("isCompleted", false);
            });
        });
    }

    function stopPropagation(e) {
        if (e && e.stopPropagation) {
            e.stopPropagation();
        }
    }
})(window.app = window.app || {})